import React, {Component} from 'react'
import PropTypes from 'prop-types'
import { fromJS } from 'immutable'
import { Form, Row, Col, Input, Select, Radio, Tooltip, InputNumber, Cascader, Checkbox, Typography } from 'antd'
import { QuestionCircleOutlined } from '@ant-design/icons'
import { formRule } from '@/utils/option.js'

import asyncComponent from '@/utils/asyncComponent'
import KeyInterface from '@/components/keyInterface'
import './index.scss'

const { TextArea } = Input
const { Paragraph } = Typography
const MkEditIcon = asyncComponent(() => import('@/components/mkIcon'))
const MKTable = asyncComponent(() => import('@/components/normalform/modalform/mkTable'))
const acTyOptions = {
  pop: ['label', 'OpenType', 'intertype', 'Ot', 'show', 'swipe', 'icon', 'class', 'color', 'execSuccess', 'execError', 'syncComponent', 'switchTab', 'anchors', 'width', 'openmenu', 'refreshTab', 'position', 'tipTitle', 'hidden'],
  prompt: ['label', 'OpenType', 'intertype', 'Ot', 'show', 'swipe', 'icon', 'class', 'color', 'execSuccess', 'execError', 'syncComponent', 'switchTab', 'anchors', 'width', 'openmenu', 'refreshTab', 'position', 'tipTitle', 'hidden'],
  exec: ['label', 'OpenType', 'intertype', 'Ot', 'show', 'swipe', 'icon', 'class', 'color', 'execSuccess', 'execError', 'syncComponent', 'switchTab', 'anchors', 'width', 'openmenu', 'refreshTab', 'hidden'],
  excelIn: ['label', 'Ot', 'OpenType', 'intertype', 'show', 'icon', 'class', 'color', 'sheet', 'execSuccess', 'execError', 'syncComponent', 'switchTab', 'width', 'hidden'],
  excelOut: ['label', 'Ot', 'OpenType', 'intertype', 'show', 'icon', 'class', 'color', 'execSuccess', 'execError', 'syncComponent', 'switchTab', 'pagination', 'search', 'width', 'hidden'],
  popview: ['label', 'Ot', 'OpenType', 'show', 'icon', 'class', 'color', 'popClose', 'width', 'display', 'ratio', 'syncComponent', 'clickouter', 'maskStyle', 'closeButton', 'hidden'],
  tab: ['label', 'Ot', 'OpenType', 'show', 'icon', 'class', 'color', 'linkmenu', 'width', 'hidden', 'openTab'],
  innerpage: ['label', 'Ot', 'OpenType', 'pageTemplate', 'show', 'swipe', 'icon', 'class', 'color', 'width', 'hidden'],
  funcbutton: ['label', 'OpenType', 'funcType', 'show', 'swipe', 'icon', 'class', 'color', 'width', 'hidden'],
  form: ['label', 'OpenType', 'formType', 'intertype', 'Ot', 'execSuccess', 'execError', 'syncComponent', 'width', 'refreshTab', 'title', 'hidden']
}

class ActionForm extends Component {
  static propTpyes = {
    type: PropTypes.any,         // type为"card"时，只可选单行或不选行
    setting: PropTypes.object,   // 页面设置
    formlist: PropTypes.any,     // 表单信息
    card: PropTypes.any,         // 按钮信息
    inputSubmit: PropTypes.func, // 回车提交事件
    updRecord: PropTypes.func    // 更新信息
  }

  state = {
    formlist: null,  // 表单信息
    appType: sessionStorage.getItem('appType'),
    requireOptions: [],
    insertUpdateOptions: [{
      value: 'insert',
      text: '添加'
    }, {
      value: 'update',
      text: '修改'
    }, {
      value: 'audit',
      text: '审核'
    }],
    deleteOptions: [{
      value: 'LogicDelete',
      text: '逻辑删除'
    }, {
      value: 'delete',
      text: '物理删除'
    }, {
      value: 'custom',
      text: '自定义'
    }],
    interTypeOptions: [{
      value: 'system',
      text: '系统'
    }, {
      value: 'inner',
      text: '内部'
    }, {
      value: 'outer',
      text: '外部'
    }, {
      value: 'custom',
      text: '自定义'
    }]
  }

  record = {}
  
  UNSAFE_componentWillMount () {
    const { type } = this.props

    let requireOptions = [
      { value: 'notRequired', text: '不选择行' },
      { value: 'requiredSgl', text: '选择单行' },
      { value: 'required', text: '选择多行' },
      { value: 'requiredOnce', text: '多行拼接' }
    ]

    if (type === 'card') {
      requireOptions = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))
    }

    let hasclass = true
    this.props.formlist.forEach(item => {
      if (!item.key) return

      this.record[item.key] = item.initVal

      if (item.key === 'class') {
        hasclass = item.forbid !== true
      }
    })

    let { shows, reOptions, reTypes, reTooltip, reRequired, reReadonly } = this.getMutilOptions(requireOptions)

    this.setState({
      hasclass,
      requireOptions: requireOptions,
      formlist: this.props.formlist.map(item => {
        item.hidden = !shows.includes(item.key)
        item.initVal = this.record[item.key]

        if (reOptions[item.key]) {
          item.options = reOptions[item.key]
        }
        if (reTypes[item.key]) {
          item.type = reTypes[item.key]
        }
        if (reTooltip[item.key] !== undefined) {
          item.tooltip = reTooltip[item.key]
        }
        if (reRequired[item.key] !== undefined) {
          item.required = reRequired[item.key]
        }
        if (reReadonly[item.key] !== undefined) {
          item.readonly = reReadonly[item.key]
        }

        return item
      })
    })
  }

  componentDidMount () {
    const { card } = this.props

    if (card.focus) {
      setTimeout(() => {
        try {
          let _form = document.getElementById('label')
          _form && _form.select()
        } catch (e) {
          console.warn('表单focus失败！')
        }
      }, 100)
    }
    this.props.updRecord && this.props.updRecord(this.record)
  }

  getMutilOptions = (requireOptions) => {
    const { appType } = this.state

    let openType = this.record.OpenType
    let Ot = this.record.Ot
    let shows = fromJS(acTyOptions[openType] || []).toJS()
    let reOptions = {}
    let reTypes = {}
    let reTooltip = {}
    let reRequired = {}
    let reReadonly = {}

    if (openType === 'pop' || openType === 'prompt' || openType === 'exec') {
      let intertype = this.record.intertype

      reOptions.intertype = this.state.interTypeOptions

      if (intertype === 'custom') {
        shows.push('procMode', 'interface', 'callbackType', 'proInterface', 'method', 'cross', 'stringify', 'ContentType', 'outerBlacklist')
        if (this.record.procMode === 'system') {
          shows.push('sql', 'sqlType')
        } else if (this.record.procMode === 'inner') {
          reRequired.innerFunc = true
          shows.push('innerFunc')
        }
        if (this.record.callbackType === 'func') {
          shows.push('callbackFunc', 'output')
        } else if (this.record.callbackType === 'script') {
          shows.push('cbTable', 'output')
        } else if (this.record.callbackType !== 'none') {
          shows.push('cbTable')
        }
        reReadonly.interface = false
        reRequired.interface = true
      } else if (intertype === 'outer') {
        shows.push('procMode', 'sysInterface', 'outerFunc', 'callbackType')
        if (this.record.procMode === 'system') {
          shows.push('sql', 'sqlType')
        } else if (this.record.procMode === 'inner') {
          reRequired.innerFunc = true
          shows.push('innerFunc')
        }
        if (this.record.callbackType === 'func') {
          shows.push('callbackFunc', 'output')
        } else if (this.record.callbackType === 'script') {
          shows.push('cbTable', 'output')
        } else if (this.record.callbackType !== 'none') {
          shows.push('cbTable')
        }

        reRequired.outerFunc = false
        if (this.record.sysInterface === 'false') {
          reReadonly.interface = false
          reRequired.interface = true

          shows.push('interface', 'proInterface')
        } else if (this.record.sysInterface === 'true') {
          reReadonly.interface = true
          reRequired.interface = false
          reRequired.outerFunc = true

          shows.push('interface')
        } else if (this.record.sysInterface === 'external') {
          shows.push('exInterface', 'exProInterface')
        }
      } else if (intertype === 'inner') {
        shows.push('innerFunc', 'output')
        if (Ot === 'requiredOnce') { // 前置函数
          shows.push('preFunc')
        }
        reRequired.innerFunc = true
      } else {
        shows.push('sql', 'sqlType', 'output')
      }

      if (Ot === 'required') {
        shows.push('progress')
      }
      if (Ot === 'required' && (intertype === 'inner' || intertype === 'system')) {
        shows.push('execType')
      }
      if (this.record.openmenu && this.record.openmenu !== 'goback') {
        shows.push('open')
      }
      if (this.record.execSuccess === 'grid' || this.record.execError === 'grid') {
        shows.push('resetPageIndex')
      }

      if (this.record.sqlType === 'insert') {
        reOptions.Ot = requireOptions.filter(op => op.value === 'notRequired')
      } else {
        reOptions.Ot = requireOptions
      }

      if (openType === 'prompt' || openType === 'exec') {
        reOptions.sqlType = this.state.deleteOptions
      } else {
        reOptions.sqlType = this.state.insertUpdateOptions
      }
      // if (this.record.execSuccess === 'goback') {
        shows.push('reload')
      // }
    } else if (openType === 'form') {
      let intertype = this.record.intertype

      if (this.record.formType === 'switch') {
        shows.push('field', 'size', 'openVal', 'closeVal', 'openText', 'closeText')
      } else if (this.record.formType === 'counter' || this.record.formType === 'count_line') {
        shows.push('field', 'size', 'min', 'max', 'decimal')
      } else if (this.record.formType === 'radio') {
        shows.push('field', 'checkType', 'openVal', 'closeVal')
      } else {
        shows.push('field')
      }

      reOptions.intertype = this.state.interTypeOptions.filter(item => item.value !== 'custom')

      if (intertype === 'outer') {
        shows.push('procMode', 'sysInterface', 'outerFunc', 'callbackType')
        if (this.record.procMode === 'system') {
          shows.push('sql', 'sqlType')
        } else if (this.record.procMode === 'inner') {
          reRequired.innerFunc = true
          shows.push('innerFunc')
        }
        if (this.record.callbackType === 'func') {
          shows.push('callbackFunc')
        } else if (this.record.callbackType !== 'none') {
          shows.push('cbTable')
        }

        reRequired.outerFunc = false
        if (this.record.sysInterface === 'false') {
          reReadonly.interface = false
          reRequired.interface = true

          shows.push('interface', 'proInterface')
        } else if (this.record.sysInterface === 'true') {
          reReadonly.interface = true
          reRequired.interface = false
          reRequired.outerFunc = true

          shows.push('interface')
        } else if (this.record.sysInterface === 'external') {
          shows.push('exInterface', 'exProInterface')
        }
      } else if (intertype === 'inner') {
        shows.push('innerFunc')
        reRequired.innerFunc = true
      } else {
        shows.push('sql', 'sqlType')
      }

      if (this.record.execSuccess === 'goback') {
        shows.push('reload')
      }
      if (this.record.execSuccess === 'grid' || this.record.execError === 'grid') {
        shows.push('resetPageIndex')
      }

      reOptions.Ot = requireOptions
      reOptions.sqlType = [{
        value: 'update',
        text: '修改'
      }, {
        value: 'custom',
        text: '自定义'
      }]

      if (this.record.formType === 'count_line') {
        reOptions.Ot = requireOptions.filter(op => ['requiredSgl'].includes(op.value))
        shows = shows.filter(op => ['label', 'OpenType', 'formType', 'Ot', 'width', 'title', 'hidden', 'field', 'size', 'min', 'max', 'decimal'].includes(op))
      }
    } else if (openType === 'excelIn') {
      reOptions.intertype = this.state.interTypeOptions.filter(op => op.value !== 'custom')
      reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))

      if (this.record.intertype === 'outer') {
        shows.push('innerFunc', 'sysInterface', 'outerFunc', 'callbackFunc')
        reRequired.innerFunc = false
        reRequired.callbackFunc = false
        reRequired.outerFunc = false

        if (this.record.sysInterface === 'false') {
          reReadonly.interface = false
          reRequired.interface = true

          shows.push('interface', 'proInterface')
        } else if (this.record.sysInterface === 'true') {
          reReadonly.interface = true
          reRequired.interface = false
          reRequired.outerFunc = true

          shows.push('interface')
        } else if (this.record.sysInterface === 'external') {
          shows.push('exInterface', 'exProInterface')
        }
      } else if (this.record.intertype === 'inner') {
        shows.push('innerFunc')
        reRequired.innerFunc = true
      }
      if (this.record.execSuccess === 'grid' || this.record.execError === 'grid') {
        shows.push('resetPageIndex')
      }
    } else if (openType === 'excelOut') {
      reOptions.intertype = this.state.interTypeOptions.filter(op => op.value !== 'custom')
      reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredOnce'].includes(op.value))

      if (this.record.intertype === 'outer') {
        shows.push('innerFunc', 'sysInterface', 'outerFunc')
        reRequired.innerFunc = false
        reRequired.outerFunc = false

        if (this.record.sysInterface === 'false') {
          reReadonly.interface = false
          reRequired.interface = true

          shows.push('interface', 'proInterface')
        } else if (this.record.sysInterface === 'true') {
          reReadonly.interface = true
          reRequired.interface = false
          reRequired.outerFunc = true

          shows.push('interface')
        } else if (this.record.sysInterface === 'external') {
          shows.push('exInterface', 'exProInterface')
        }
      } else if (this.record.intertype === 'inner') {
        shows.push('innerFunc')
        reRequired.innerFunc = true
      }
      if (this.record.execSuccess === 'grid' || this.record.execError === 'grid') {
        shows.push('resetPageIndex')
      }
      if (this.record.Ot !== 'notRequired' && this.record.Ot !== 'requiredOnce') {
        this.record.Ot = 'notRequired'
      }
    } else if (openType === 'popview') {
      reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))

      if (appType === 'mob') {
        shows.push('placement')
      } else if (this.record.display === 'drawer') {
        shows.push('placement')
      } else if (this.record.display === 'modal') {
        shows.push('popshow')
      }
      if (this.record.popClose === 'grid') {
        shows.push('resetPageIndex')
      }
    } else if (openType === 'tab') {
      reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))

      reRequired.linkmenu = true
      reTooltip.linkmenu = ''
    } else if (openType === 'innerpage') {
      reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))

      if (this.record.pageTemplate === 'custom') {
        shows.push('url', 'proUrl', 'joint', 'open')
      } else if (this.record.pageTemplate === 'linkpage') {
        shows.push('linkmenu', 'open')

        if (Ot === 'requiredSgl') {
          shows.push('joint')
        }

        reRequired.linkmenu = true
        reTooltip.linkmenu = ''
      } else if (this.record.pageTemplate === 'billprint') {
        shows.push('printTemp')
        reOptions.Ot = requireOptions
      } else if (this.record.pageTemplate === 'pay') {
        reOptions.Ot = requireOptions.filter(op => op.value === 'requiredSgl')
      }
    } else if (openType === 'funcbutton') {
      reOptions.intertype = this.state.interTypeOptions.filter(op => op.value !== 'custom')
      reOptions.Ot = requireOptions

      let _funcType = this.record.funcType

      if (_funcType === 'print') {
        shows.push('execMode', 'intertype', 'Ot', 'execSuccess', 'execError')
        if (this.record.intertype === 'outer') {
          shows.push('innerFunc', 'sysInterface', 'outerFunc', 'callbackFunc')
          reRequired.innerFunc = false
          reRequired.callbackFunc = false
          reRequired.outerFunc = false

          if (this.record.sysInterface === 'false') {
            reReadonly.interface = false
            reRequired.interface = true
  
            shows.push('interface', 'proInterface')
          } else if (this.record.sysInterface === 'true') {
            reReadonly.interface = true
            reRequired.interface = false
            reRequired.outerFunc = true
  
            shows.push('interface')
          } else if (this.record.sysInterface === 'external') {
            shows.push('exInterface', 'exProInterface')
          }
        } else if (this.record.intertype === 'inner') {
          shows.push('innerFunc')
          reRequired.innerFunc = true
        }
        if (this.record.execSuccess === 'grid' || this.record.execError === 'grid') {
          shows.push('resetPageIndex')
        }
        if (this.record.execMode === 'pop' || this.record.execMode === 'prompt') {
          shows.push('position')
        }
      } else if (_funcType === 'mkBinding' || _funcType === 'mkUnBinding') {
        shows.push('execSuccess', 'execError')
      } else if (_funcType === 'closetab') {
        shows.push('refreshTab')
      } else if (_funcType === 'scan') {
        shows.push('linkmenu', 'prefix')

        reRequired.linkmenu = false
        reTooltip.linkmenu = '使用扫码登录功能或菜单跳转功能时，需选择跳转的菜单。'
      } else if (_funcType === 'copyurl') {
        shows.push('linkmenu')

        reRequired.linkmenu = false
        reTooltip.linkmenu = '可以指定分享的目标地址。'

        if (this.record.linkmenu) {
          shows.push('Ot')
          reOptions.Ot = requireOptions.filter(op => ['notRequired', 'requiredSgl'].includes(op.value))
        }
      } else if (_funcType === 'goBack') {
        shows.push('reload')
      } else if (_funcType === 'megvii') {
        shows.push('subFunc', 'progress')
      } else if (_funcType === 'filezip') {
        reOptions.Ot = requireOptions
        reRequired.innerFunc = false

        shows.push('innerFunc', 'Ot', 'execSuccess', 'execError', 'urlkey')
      } else if (_funcType === 'pay') {
        shows.push('payType', 'Ot', 'execSuccess', 'execError', 'syncComponent', 'openmenu')
        if (this.record.openmenu && this.record.openmenu !== 'goback') {
          shows.push('open')
        }
        reOptions.Ot = requireOptions.filter(op => ['requiredSgl'].includes(op.value))
      } else if (_funcType === 'refund') {
        shows.push('Ot', 'execSuccess', 'execError', 'syncComponent', 'openmenu')
        reOptions.Ot = requireOptions.filter(op => ['requiredSgl'].includes(op.value))
      }
    }
    
    if (appType === 'mob') {
      if (openType !== 'excelOut') {
        shows.push('control')
        reOptions.control = [
          { value: '', text: '无' },
          { value: 'disabled', text: '禁用' },
          { value: 'hidden', text: '隐藏' },
        ]
        if (this.record.control) {
          shows.push('controlField', 'controlVal')
        }
        if (this.record.control === 'disabled') {
          shows.push('reason')
        }
      }
    } else {
      if (openType !== 'excelOut') {
        reOptions.control = [
          { value: '', text: '无' },
          { value: 'disabled', text: '禁用' },
          { value: 'hidden', text: '隐藏' },
          { value: 'parent', text: '上级' }
        ]
      } else {
        reOptions.control = [
          { value: '', text: '无' },
          { value: 'parent', text: '上级' }
        ]
      }
      shows.push('control')
      if (this.record.control === 'parent') {
        reTypes.controlField = 'text'
      } else {
        reTypes.controlField = 'select'
      }
      if (this.record.control) {
        shows.push('controlField', 'controlVal')
      }
      if (this.record.control === 'disabled') {
        shows.push('reason')
      }
    }

    if (shows.includes('syncComponent') && this.record.syncComponent[0] === 'multiComponent') {
      shows.push('syncComponents')
    }

    if (this.record.show === 'icon') {
      reRequired.icon = true
    } else {
      reRequired.icon = false
    }

    return {
      shows,
      reOptions,
      reTypes,
      reTooltip,
      reRequired,
      reReadonly
    }
  }

  /**
   * @description 下拉切换
   * 1、打开方式切换，重置可见表单和表单值
   * 2、显示位置切换，重置选择行
   * 3、切换标签类型，重置可选标签
   */
  optionChange = (key, value) => {
    const { hasclass, appType, requireOptions } = this.state

    this.record[key] = value
    let _fieldval = {}

    if (key === 'OpenType') {
      this.record.intertype = 'system'
      this.record.procMode = 'system'
      this.record.display = 'modal'
      this.record.pageTemplate = ''
      this.record.funcType = ''
      this.record.sqlType = ''
      if (value === 'pop' || value === 'prompt' || value === 'exec') {
        _fieldval.intertype = 'system'
        _fieldval.sqlType = ''
      } else if (value === 'form') {
        _fieldval.Ot = 'requiredSgl'

        this.record.Ot = 'requiredSgl'
      } else if (value === 'excelIn') {
        _fieldval.intertype = 'system'
        _fieldval.Ot = 'notRequired'
        _fieldval.execSuccess = 'grid'
        _fieldval.label = '导入Excel'
        _fieldval.class = 'border-dgreen'
        this.record.Ot = 'notRequired'
        this.record.label = '导入Excel'
        this.record.class = 'border-dgreen'
        this.record.execSuccess = 'grid'
      } else if (value === 'excelOut') {
        _fieldval.intertype = 'system'
        _fieldval.label = '导出Excel'
        _fieldval.class = 'dgreen'
        _fieldval.execSuccess = 'never'
        _fieldval.Ot = 'requiredOnce'
        _fieldval.control = ''
        this.record.Ot = 'requiredOnce'
        this.record.label = '导出Excel'
        this.record.class = 'dgreen'
        this.record.execSuccess = 'never'
        this.record.control = ''
      } else if (value === 'popview') {
        _fieldval.display = 'modal'
        _fieldval.Ot = 'requiredSgl'
        _fieldval.popClose = 'grid'
        this.record.display = 'modal'
        this.record.Ot = 'requiredSgl'
        this.record.popClose = 'grid'
  
      } else if (value === 'tab') {
        _fieldval.Ot = 'requiredSgl'
        this.record.Ot = 'requiredSgl'
  
      } else if (value === 'innerpage') {
        _fieldval.Ot = 'requiredSgl'
        this.record.Ot = 'requiredSgl'
        
      } else if (value === 'funcbutton') {
        // _fieldval.funcType = ''
  
      }

      if (appType !== 'mob' && _fieldval.Ot === 'notRequired') {
        this.record.control = ''
        _fieldval.control = ''
      }

      this.props.updRecord && this.props.updRecord(this.record)
    } else if (key === 'formType') {
      if (value === 'count_line') {
        _fieldval.Ot = 'requiredSgl'
        this.record.Ot = 'requiredSgl'
      }
    } else if (key === 'funcType') {
      if (value === 'print') {
        _fieldval.label = '打印'
      } else if (value === 'closetab') {
        _fieldval.label = '关闭'
      }
    } else if (key === 'sqlType') {
      if (value === 'insert') {
        _fieldval.label = '添加'
        _fieldval.Ot = 'notRequired'
        if (hasclass) {
          _fieldval.class = 'green'
        } else {
          _fieldval.color = 'success'
        }
      } else if (value === 'update') {
        _fieldval.label = '修改'
        _fieldval.Ot = 'requiredSgl'
        if (this.record.OpenType !== 'form') {
          if (hasclass) {
            _fieldval.class = 'purple'
          } else {
            _fieldval.color = 'primary'
          }
        }
      } else if (value === 'audit') {
        _fieldval.label = '审核'
        _fieldval.Ot = 'required'
        if (hasclass) {
          _fieldval.class = 'primary'
        } else {
          _fieldval.color = 'warning'
        }
      } else if (value === 'LogicDelete' || value === 'delete') {
        _fieldval.label = '删除'
        _fieldval.Ot = 'required'
        if (hasclass) {
          _fieldval.class = 'danger'
        } else {
          _fieldval.color = 'danger'
        }
      } else if (value === 'custom') {
        _fieldval.Ot = 'required'
      }

      if (this.props.type === 'card' && _fieldval.Ot === 'required') { // 卡片中无多行选项
        _fieldval.Ot = 'requiredSgl'
      }

      this.record.label = _fieldval.label || this.record.label || ''
      this.record.Ot = _fieldval.Ot || this.record.Ot || ''
      this.record.class = _fieldval.class || this.record.class || ''
      this.record.color = _fieldval.color || this.record.color || ''

      if (appType !== 'mob' && _fieldval.Ot === 'notRequired') {
        this.record.control = ''
        _fieldval.control = ''
      }
    } else if (key === 'Ot') {
      if (appType !== 'mob' && value === 'notRequired') {
        this.record.control = ''
        _fieldval.control = ''
      }
    } else if (key === 'pageTemplate') {
      if (value === 'pay') {
        _fieldval.Ot = 'requiredSgl'
        this.record.Ot = 'requiredSgl'
      }
    } else if (key === 'intertype') {
      this.record.sysInterface = 'false'
      
      this.props.updRecord && this.props.updRecord(this.record)
    } else if (key === 'sysInterface') {
      if (value === 'true') {
        _fieldval.interface = window.GLOB.mainSystemApi || ''
        this.record.interface = window.GLOB.mainSystemApi || ''
      }
    }

    let { shows, reOptions, reTypes, reTooltip, reRequired, reReadonly } = this.getMutilOptions(requireOptions)

    this.setState({
      formlist: this.state.formlist.map(item => {
        item.hidden = !shows.includes(item.key)
        item.initVal = this.record[item.key]

        if (reOptions[item.key]) {
          item.options = reOptions[item.key]
        }
        if (reTypes[item.key]) {
          item.type = reTypes[item.key]
        }
        if (reTooltip[item.key] !== undefined) {
          item.tooltip = reTooltip[item.key]
        }
        if (reRequired[item.key] !== undefined) {
          item.required = reRequired[item.key]
        }
        if (reReadonly[item.key] !== undefined) {
          item.readonly = reReadonly[item.key]
        }

        return item
      })
    }, () => {
      this.props.form.setFieldsValue(_fieldval)
    })
  }

  handleSubmit = (e) => {
    e.preventDefault()

    if (this.props.inputSubmit) {
      this.props.inputSubmit()
    }
  }

  getFields() {
    const { getFieldDecorator } = this.props.form
    const fields = []

    this.state.formlist.forEach((item, index) => {
      if (item.hidden || item.forbid) return

      let span = 12
      let rules = []
      let className = ''
      let content = null
      let initVal = item.initVal || ''

      if (item.type === 'splitLine') {
        fields.push(
          <Col span={24} key={index}>
            <p style={{borderBottom: '1px solid #e9e9e9', color: '#1890ff', textAlign: 'center'}}>{item.label}</p>
          </Col>
        )
        return
      }
      if (item.type === 'text') {
        rules = [
          { required: item.required, message: '请输入' + item.label + '!' }
        ]
  
        if (item.key === 'innerFunc') {
          let str = item.fields && item.fields.length ? '^(' + item.fields.join('|') + ')' : '^'
          let _patten = new RegExp(str + formRule.func.innerPattern + '$', 'g')
          rules.push(
            { pattern: _patten, message: formRule.func.innerMessage },
            { max: formRule.func.max, message: formRule.func.maxMessage }
          )
        } else if (item.key === 'sql' || item.key === 'sheet') {
          rules.push(
            { pattern: /^[a-zA-Z0-9@_]+$/, message: '表名只可使用字母、数字以及_' },
            { max: formRule.func.max, message: formRule.func.maxMessage }
          )
        } else if (item.key === 'outerFunc' || item.key === 'callbackFunc' || item.key === 'preFunc') {
          rules.push(
            { pattern: formRule.func.pattern, message: formRule.func.message },
            { max: formRule.func.max, message: formRule.func.maxMessage }
          )
        } else if (item.key === 'outerBlacklist') {
          rules.push(
            { max: 512, message: '最多512个字符' }
          )
        } else if (item.key === 'output') {
          if (this.record.intertype === 'system' || ((this.record.intertype === 'outer' || this.record.intertype === 'custom') && this.record.callbackType === 'script')) {
            rules = [{
              pattern: /^@[0-9a-zA-Z_]+@?$/,
              message: '变量以@符开头，可使用字母、数字以及_'
            }, {
              max: 100,
              message: '最多100个字符。'
            }]
          } else {
            rules = [{
              pattern: /^[0-9a-zA-Z_]*$/,
              message: '字段可使用字母、数字以及_'
            }, {
              max: 100,
              message: '最多100个字符。'
            }]
          }
        } else {
          rules.push({ max: formRule.input.max, message: formRule.input.message })
        }

        content = <Input placeholder="" autoComplete="off" onPressEnter={this.handleSubmit} />
      } else if (item.type === 'number') {
        initVal = item.initVal || item.initVal === 0 ? item.initVal : ''
        rules = [
          { required: item.required, message: '请输入' + item.label + '!' }
        ]

        content = <InputNumber min={item.min} max={item.max} precision={item.precision} onPressEnter={this.handleSubmit}/>
      } else if (item.type === 'select') {
        rules = [
          { required: item.required, message: '请选择' + item.label + '!' }
        ]

        if (item.extendName) {
          content = <Select
            showSearch
            allowClear={item.allowClear === true || !item.required}
            filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0 ||
              option.props.extend.toLowerCase().indexOf(input.toLowerCase()) >= 0}
            onChange={(value) => {this.optionChange(item.key, value)}}
            getPopupContainer={() => document.getElementById('winter')}
          >
            {item.options.map((option, index) =>
              <Select.Option key={index} title={option.text || option.label} extend={option[item.extendName] || ''} value={(option.value || option.field)}>
                {(option.text || option.label)}
              </Select.Option>
            )}
          </Select>
        } else {
          content = <Select
            showSearch
            allowClear={item.allowClear === true || !item.required}
            filterOption={(input, option) => option.props.children.toLowerCase().indexOf(input.toLowerCase()) >= 0}
            onChange={(value) => {this.optionChange(item.key, value)}}
            getPopupContainer={() => document.getElementById('winter')}
          >
            {item.options.map((option, index) =>
              <Select.Option key={index} title={option.text || option.label} value={(option.value || option.field)}>
                {(option.text || option.label)}
              </Select.Option>
            )}
          </Select>
        }
      } else if (item.type === 'radio') {
        rules = [
          { required: item.required, message: '请选择' + item.label + '!' }
        ]

        content = <Radio.Group onChange={(e) => {this.optionChange(item.key, e.target.value)}}>
          {item.options.map(option => <Radio key={option.value} value={option.value}>{option.text}</Radio>)}
        </Radio.Group>
      } else if (item.type === 'checkbox') {
        rules = [
          { required: item.required, message: '请选择' + item.label + '!' }
        ]

        content = <Checkbox.Group>
          {item.options.map(option => <Checkbox key={option.value} value={option.value}>{option.text}</Checkbox>)}
        </Checkbox.Group>
      } else if (item.type === 'cascader') {
        initVal = item.initVal || []
        rules = [
          { required: item.required, message: '请选择' + item.label + '!' }
        ]

        content = <Cascader onChange={(value) => {this.optionChange(item.key, value)}} options={item.options || []} expandTrigger="hover" placeholder=""/>
      } else if (item.type === 'table') {
        span = 24
        className = 'textarea'
        rules = [
          { required: item.required, message: '请添加' + item.label + '!' }
        ]
        content = (<MKTable tip={''} columns={item.columns || []} actions={[]}/>)
      } else if (item.type === 'icon') {
        rules = [
          { required: item.required, message: '请选择' + item.label + '!' }
        ]

        content = <MkEditIcon options={['edit', 'hint', 'direction', 'normal', 'data']} allowClear/>
      } else if (item.type === 'textarea') {
        span = 24
        className = 'textarea'
        rules = [
          { required: item.readonly ? false : item.required, message: '请输入' + item.label + '!' }
        ]

        if (item.key === 'url' || item.key === 'proUrl') {
          rules.push({
            pattern: /^[^\s]*$/,
            message: '地址中不可使用空格！'
          })
        }

        content = <TextArea rows={2} readOnly={item.readonly}/>
      } else if (item.type === 'keyinterface') {
        span = 24
        className = 'textarea'
        rules = [
          { required: item.required, message: '请输入' + item.label + '!' }
        ]

        content = <KeyInterface type={item.key === 'exInterface' ? 'develop' : 'product'}/>
      }

      fields.push(
        <Col span={span} key={index}>
          <Form.Item className={className} help={item.help} label={item.tooltip ?
            <Tooltip placement="topLeft" title={item.tooltip}>
              <QuestionCircleOutlined className="mk-form-tip" />
              {item.label}
            </Tooltip> : item.label
          }>
            {getFieldDecorator(item.key, {
              initialValue: initVal,
              rules: rules
            })(content)}
          </Form.Item>
        </Col>
      )
    })

    if (window.debugger && this.props.card.uuid) {
      fields.push(
        <Col span={12} key="uuid">
          <Form.Item label="按钮ID">
            <Paragraph copyable>{this.props.card.uuid}</Paragraph>
          </Form.Item>
        </Col>
      )
    }

    return fields
  }

  handleConfirm = () => {
    const { card } = this.props
    // 表单提交时检查输入值是否正确
    return new Promise((resolve, reject) => {
      this.props.form.validateFieldsAndScroll((err, values) => {
        if (!err) {
          values.uuid = card.uuid
          values.verify = card.verify || null
          values.modal = card.modal || null
          values.config = card.config || null
          values.$fixed = card.$fixed || false

          if (card.OpenType === 'excelOut' && values.OpenType === 'excelIn') {
            if (values.verify && values.verify.columns && values.verify.columns.length > 0) {
              values.verify.columns = values.verify.columns.map(col => {
                col.required = col.required || 'true'
                col.type = col.type || 'Nvarchar(50)'
                col.import = col.import || 'true'
          
                if (col.type === 'text' || col.type === 'image') {
                  col.type = 'Nvarchar(50)'
                } else if (col.type === 'number') {
                  col.type = 'Decimal(18,2)'
                }
                
                if (/^Nvarchar/ig.test(col.type)) {
                  col.limit = col.type.match(/\d+/)[0]
                } else if (/^Decimal/ig.test(col.type)) {
                  col.limit = col.type.match(/\d+/ig)[1]
                } else {
                  col.limit = ''
                }

                delete col.output
                delete col.abs
                delete col.Width
          
                return col
              })

              values.verify.sheet = values.verify.sheet || 'Sheet1'
            }
          } else if (card.OpenType === 'excelIn' && values.OpenType === 'excelOut') {
            if (values.verify && values.verify.columns && values.verify.columns.length > 0) {
              values.verify.columns = values.verify.columns.map(col => {
                col.type = col.type || 'text'
                col.output = col.output || 'true'
                col.required = col.required || 'false'
                col.Width = 20
          
                if (!['text', 'image', 'number'].includes(col.type)) {
                  if (/^Decimal/ig.test(col.type)) {
                    col.type = 'number'
                  } else {
                    col.type = 'text'
                  }
                }

                delete col.import
          
                return col
              })
            }
          }

          if (values.OpenType === 'form') {
            if (values.formType !== 'scan') {
              if (/^(0|[1-9]\d*)$/.test(values.openVal) && /^(0|[1-9]\d*)$/.test(values.closeVal)) {
                values.openVal = +values.openVal
                values.closeVal = +values.closeVal
              }
            } else {
              values.show = 'icon'
              values.icon = 'scan'
            }
          } else if (values.OpenType === 'excelOut') {
            values.Ot = values.Ot || 'notRequired'
          } else if (['pop', 'prompt', 'exec'].includes(values.OpenType) && values.verify) {
            if ((values.Ot === 'requiredOnce' || card.Ot === 'requiredOnce') && card.Ot !== values.Ot) {
              values.verify.uniques = []
            }
            if (card.Ot !== values.Ot) {
              if (values.Ot === 'notRequired') {
                values.verify.invalid = 'false'
              } else if (card.Ot === 'notRequired' && values.Ot !== 'notRequired') {
                values.verify.invalid = 'true'
              }
            }
          } else if (values.OpenType === 'tab' && values.linkmenu && Array.isArray(values.linkmenu)) {
            if (sessionStorage.getItem('thdMenuList')) {
              let list = null
              try {
                list = JSON.parse(sessionStorage.getItem('thdMenuList')) || []
              } catch (e) {
                list = []
              }
        
              let id = values.linkmenu[values.linkmenu.length - 1]
        
              list.forEach(item => {
                if (item.MenuID === id) {
                  values.MenuID = id
                  values.MenuName = item.MenuName
                  values.MenuNo = item.MenuNo
                  values.tabType = item.type
                }
              })
            }
          }
          
          if (values.outerBlacklist) {
            values.outerBlacklist = values.outerBlacklist.replace(/\s/ig, '')
          }

          if (values.openmenu && Array.isArray(values.openmenu) && values.openmenu.length > 0) {
            let list = null
            try {
              list = JSON.parse(sessionStorage.getItem('thdMenuList')) || []
            } catch (e) {
              list = []
            }

            let id = values.openmenu[values.openmenu.length - 1]

            list.forEach(item => {
              if (item.MenuID === id) {
                values.MenuID = id
                values.MenuName = item.MenuName
                values.MenuNo = item.MenuNo
                values.tabType = item.type
              }
            })
          }

          resolve(values)
        } else {
          reject(err)
        }
      })
    })
  }

  render() {
    const formItemLayout = {
      labelCol: {
        xs: { span: 24 },
        sm: { span: 7 }
      },
      wrapperCol: {
        xs: { span: 24 },
        sm: { span: 17 }
      }
    }
    return (
      <Form {...formItemLayout} className="menu-action-list-form" id="winter">
        <Row gutter={24}>{this.getFields()}</Row>
      </Form>
    )
  }
}

export default Form.create()(ActionForm)